import { d3 } from '../d3-bundle';
import { TimeInterval } from '../interfaces/TimeInterval';
import { ChartDataService } from './chart-data.service';
const dayInMs = 1000 * 60 * 60 * 24;
export class ChartService {
    static addArea(svg, line, y, data, color) {
        const area = d3.area()
            .curve(d3.curveMonotoneX)
            .defined(line.defined())
            .x(line.x())
            .y1(line.y())
            .y0(y(0));
        svg.append('path')
            .data([data])
            .attr('class', 'series area')
            .attr('fill', color)
            .attr('d', area);
        return area;
    }
    static addLine(svg, x, y, data, color) {
        const valueLine = d3.line()
            .curve(d3.curveMonotoneX)
            .defined((d) => d.y !== null)
            .x((d) => x(d.x))
            .y((d) => y(d.y));
        svg.append('path')
            .data([data])
            .attr('class', 'line')
            .attr('stroke', color)
            .attr('d', valueLine);
        return valueLine;
    }
    static addOverlay(svg, dims) {
        const overlay = svg.append('rect')
            .attr('class', 'overlay')
            .attr('width', dims.width)
            .attr('height', dims.height);
        return overlay;
    }
    static addPoints(svg, x, y, data, color) {
        svg.selectAll('.point')
            .data(data)
            .enter()
            .append('svg:circle')
            .style('opacity', 0)
            .style('stroke', color)
            .attr('class', (_d, i) => `point point-${i}`)
            .attr('r', () => 1)
            .attr('cx', (d) => x(d.x))
            .attr('cy', (d) => y(d.y));
    }
    static addRevealAnimation(svg, dims, axisConfigs) {
        const delay = ChartService.focusLength;
        const duration = ChartService.animationLength;
        let curtainHeight = dims.height;
        let curtainWidth = dims.width;
        if (!axisConfigs.x.showAxis) {
            curtainHeight = curtainHeight + 1;
        }
        if (axisConfigs.y.showAxis) {
            curtainWidth = curtainWidth - (this.yPad * 2.5);
        }
        const curtain = svg.append('rect')
            .attr('x', -1 * dims.width)
            .attr('y', -1 * curtainHeight)
            .attr('height', curtainHeight)
            .attr('width', curtainWidth)
            .attr('class', 'curtain')
            .attr('transform', 'rotate(180)')
            .style('fill', '#ffffff');
        const t = svg.transition()
            .delay(delay)
            .duration(duration);
        t.select('rect.curtain')
            .attr('width', 0);
        t.select('line.guide')
            .attr('transform', 'translate(' + dims.width + ', 0)');
        return curtain;
    }
    static addXAxis(svg, dims, data, axisConfigs) {
        const x = d3.scaleTime()
            .domain(ChartService.determineDateRange(data))
            .nice()
            .range([this.calculateLeftPad(dims, axisConfigs), dims.width]);
        const xAxis = d3.axisBottom(x)
            .tickSizeOuter(0)
            .ticks(Math.min(this.calculateTicks(dims.width), data.length * 2))
            .tickFormat(ChartDataService.xFormatter.bind(this, ChartService.calculateInterval(data), axisConfigs.x))
            .scale(x);
        svg.append('g')
            .attr('class', 'x axis')
            .attr('transform', `translate(0,${dims.height})`)
            .call(xAxis);
        if (!axisConfigs.x.showAxis) {
            svg.select('.x.axis').style('display', 'none');
        }
        if (!axisConfigs.x.showTicks) {
            svg.selectAll('.x.axis .tick').style('display', 'none');
        }
        return x;
    }
    static addYAxis(svg, dims, data, axisConfigs) {
        const y = d3.scaleLinear()
            .domain([0, d3.max(data, (d) => d.y)])
            .nice()
            .range([dims.height, dims.margin.top]);
        const yAxis = d3.axisLeft(y)
            .tickSize(0)
            .ticks(this.calculateTicks(dims.height))
            .tickFormat(ChartDataService.yFormatter.bind(this, axisConfigs.y))
            .scale(y);
        svg.append('g')
            .attr('class', 'y axis')
            .attr('transform', `translate(${this.calculateLeftPad(dims, axisConfigs)},0)`)
            .call(yAxis);
        if (!axisConfigs.y.showAxis) {
            svg.select('.y.axis').style('display', 'none');
        }
        if (!axisConfigs.y.showTicks) {
            svg.selectAll('.y.axis .tick').style('display', 'none');
        }
        return y;
    }
    static calculateInterval(data) {
        const dayPeriod = 1000 * 60 * 60 * 24;
        let interval = TimeInterval.Monthly;
        if (data.length < 2) {
            return interval;
        }
        const date1 = new Date(data[0].x);
        const date2 = new Date(data[1].x);
        if (isNaN(date1.valueOf()) || isNaN(date2.valueOf())) {
            return interval;
        }
        const timeElapsed = date2.valueOf() - date1.valueOf();
        if (timeElapsed < dayPeriod * 2) {
            interval = TimeInterval.Daily;
        }
        else if (timeElapsed < dayPeriod * 27) {
            interval = TimeInterval.Weekly;
        }
        else if (timeElapsed > dayPeriod * 364) {
            interval = TimeInterval.Yearly;
        }
        return interval;
    }
    static createSVG(dims) {
        const svg = d3.create('svg');
        svg
            .attr('width', dims.width + dims.margin.left)
            .attr('height', dims.height + dims.margin.top)
            .attr('transform', `translate(${dims.margin.left}, ${dims.margin.top})`)
            .append('g');
        return svg;
    }
    static determineDateRange(data) {
        const domainPadding = this.domainPadding(data);
        const minDate = data.reduce((r, p) => {
            if (r === null) {
                return +p.x;
            }
            return Math.min(r, +p.x);
        }, null);
        const maxDate = data.reduce((r, p) => {
            if (r === null) {
                return +p.x;
            }
            return Math.max(r, +p.x);
        }, null);
        return [new Date(+minDate - domainPadding), new Date(+maxDate + domainPadding)];
    }
    static domainPadding(data) {
        const interval = ChartService.calculateInterval(data);
        switch (interval) {
            case TimeInterval.Daily:
                return this.halfDayMs;
            case TimeInterval.Weekly:
                return this.halfWeekMs;
            case TimeInterval.Monthly:
                return this.halfMonthMs;
            case TimeInterval.Yearly:
                return this.halfYearMs;
            default:
                return this.halfMonthMs;
        }
    }
    static getIndexOfClosestPoint(x, data) {
        const x0 = x.invert(d3.mouse(d3.event.currentTarget)[0]);
        const bisector = d3.bisector((d) => d.x).left;
        const index = bisector(data, x0, 1);
        const d0 = data[index - 1];
        const d1 = data[index];
        const rangeValueOfFirst = x((d0) ? d0.x : null), rangeValueOfSecond = x((d1) ? d1.x : null), rangeValueOfMousePos = d3.mouse(d3.event.currentTarget)[0], closestIndex = Math.abs(rangeValueOfMousePos - rangeValueOfFirst) > Math.abs(rangeValueOfMousePos - rangeValueOfSecond) ? index : index - 1;
        return closestIndex;
    }
    static hideFocus(svg) {
        svg.selectAll('.point')
            .transition()
            .duration(ChartService.focusLength)
            .attr('r', 1)
            .style('opacity', '0');
    }
    static showFocus(svg, activeIndex) {
        ChartService.hideFocus(svg);
        svg.selectAll('.point-' + activeIndex)
            .transition()
            .duration(ChartService.focusLength)
            .attr('r', 4)
            .style('opacity', '1');
    }
    static calculateLeftPad(dims, axisConfigs) {
        if (axisConfigs.y.showAxis || axisConfigs.y.showTicks) {
            return this.yPad + dims.margin.left;
        }
        else {
            return 0;
        }
    }
    static calculateTicks(dim) {
        return Math.max(dim / 100, 2);
    }
}
ChartService.colors = [
    '#384b5d',
    '#6ec5ab',
    '#6D95A0',
    '#995D81',
    '#586BA4',
    '#58A4B0',
    '#D496A7',
    '#C8AD55',
    '#8EA4D2',
    '#4ECDC4',
    '#98D2EB',
    '#857C8D'
];
ChartService.animationLength = 2500;
ChartService.focusLength = 400;
ChartService.halfDayMs = dayInMs / 2;
ChartService.halfWeekMs = dayInMs * (7 / 2);
ChartService.halfMonthMs = dayInMs * (30.42 / 2);
ChartService.halfYearMs = dayInMs * (365 / 2);
ChartService.yPad = 15;
